#include "Arduboy.h"
#include "gfx.h"
#define MAX_HEIGHT                56

Arduboy arduboy;

long int frameNumber=0;
unsigned char rowStart[16];
unsigned char snowRow[MAX_HEIGHT];

byte myFrame=0;
signed int fd=1;
int frameRate=120;
int FPS = 0;
bool drawSnow = false;

signed int playerX = 0;
bool playerDirection = 0;
byte walkingFrame=0;

// Button updates
byte input; // for capturing buttons
#define HELD 0
#define NEW 1
#define RELEASE 2
byte CompletePad, ExPad, TempPad;
bool _A[3], _B[3], _Up[3], _Down[3], _Left[3], _Right[3];

void UPDATEPAD(int pad, int var){
  _B[pad] = (var & B_BUTTON) >> 2; // 4
  _A[pad] = (var & A_BUTTON) >> 3; // 8
  _Down[pad] = (var & DOWN_BUTTON) >> 4; // 16
  _Left[pad] = (var & LEFT_BUTTON) >> 5; // 32
  _Right[pad] = (var & RIGHT_BUTTON) >> 6; // 64
  _Up[pad] = (var & UP_BUTTON) >> 7; // 128
}
void UpdatePad(int joy_code) 
{
  ExPad = CompletePad;
  CompletePad = joy_code;
  UPDATEPAD(HELD, CompletePad); // held
  UPDATEPAD(RELEASE, (ExPad & (~CompletePad))); // released
  UPDATEPAD(NEW, (CompletePad & (~ExPad))); // newpress
}

static inline void fps(const int seconds){
  // Create static variables so that the code and variables can
  // all be declared inside a function
  static unsigned long lastMillis;
  static unsigned long frameCount;
  static unsigned int framesPerSecond;
  
  // It is best if we declare millis() only once
  unsigned long now = millis();
  frameCount ++;
  if (now - lastMillis >= seconds * 1000) {
    framesPerSecond = frameCount / seconds;
    //Serial.println(framesPerSecond);
    FPS = framesPerSecond;
    frameCount = 0;
    lastMillis = now;
  }
}


void setup() {

  
  arduboy.begin();
  arduboy.initRandomSeed();
  arduboy.setFrameRate(120);
  arduboy.clear();

    for (char t = 0; t < 18; t++) {
      vert[t] = random(MAX_HEIGHT >> 1);
      if (t % 1 == 0) {
        vert[t] <<= 1;
      }
    }
    for (char t = 0; t < 16; t++) {
      rowStart[t] = (random(MAX_HEIGHT >> 1));
      if (t % 1 == 0) {
        rowStart[t] <<= 1;
      }
    }
  
}

void loop() {
  //if (!(arduboy.nextFrame())) return;

  frameNumber++;
  UpdatePad(arduboy.buttonsState());

  arduboy.clear();
  if(drawSnow){updateSnow();}



  
  
  char text[] = "                  ";
  sprintf(text, "frameRate:%d",frameRate);
  arduboy.setCursor(0, 40);
  arduboy.print(text);
  sprintf(text, "FPS:%d",FPS);
  arduboy.setCursor(0, 48);
  arduboy.print(text);


  arduboy.fillRect(0,56,128,8,WHITE);
  // draw player
  arduboy.drawBitmap(playerX, 44, spriteMask+(pgm_read_byte(&walkingFrame[walkFrame])*32)+(playerDirection*256), 16, 16, 0);
  arduboy.drawBitmap(playerX, 44, sprite+((frameNumber%2)*32)+(pgm_read_byte(&walkingFrame[walkFrame])*64)+(playerDirection*512), 16, 16, 1);


  arduboy.display();

  if(_Right[HELD]){
    playerX++;
    playerDirection=0;
    if(frameNumber%8==0){
      walkingFrame++;
      if(walkingFrame >= 8){walkingFrame=0;}
    }
  }
  if(_Left[HELD]){
    playerX--;
    playerDirection=1;
    if(frameNumber%8==0){
      walkingFrame++;
      if(walkingFrame >= 8){walkingFrame=0;}
    }
  }
  if(playerX > 128){playerX = -15;}
  if(playerX <-15){playerX = 128;}

  if(_A[RELEASE] || _Right[RELEASE] || _Left[RELEASE]){walkingFrame=2;}
  if(_A[HELD]){walkingFrame=8;}
  if(_B[NEW]){drawSnow=!drawSnow;}

  if(_Up[NEW]){frameRate++; arduboy.setFrameRate(frameRate);}
  if(_Down[NEW]){frameRate--; arduboy.setFrameRate(frameRate);}
  fps(1);
}


void updateSnow() {
  // Main snow-feild code
  unsigned char ctr = 0;
  unsigned char stp = 1;
  for (unsigned char loop = 17; loop > stp + 7; loop -= stp) { // 17
    countdn[loop]--;
    if (countdn[loop] == 0) {
      countdn[loop] = spead[loop];
      //remove old flake image
      int y = vert[loop];
      int a = 0;
      for (a = 0; a < 4; a++) {
        snowRow[y] &= (255 - tempimg[loop * 4 + a]);
        y++; if (y >= MAX_HEIGHT) {
          y = 0; // was 176
        }
      }//a

      // rem calculate new vertical position
      // rem (vert) & frame number (offset)
      vert[loop]++;
      if (vert[loop] >= MAX_HEIGHT) {
        vert[loop] = 0;
      }
      offset[loop]--;
      if (offset[loop] < 0) {
        offset[loop] = mx[loop];
      }
      // rem get next frame of animation
      y = pgm_read_byte(&frames[fptrs[loop] + offset[loop]]);
      for (a = 0; a < 4; a++) {
        tempimg[loop * 4 + a] = pgm_read_byte(&flkimage[y + a]);
      }
      // rem add new frame at new position
      y = vert[loop];
      for (a = 0; a < 4; a++) {
        snowRow[y] |= tempimg[loop * 4 + a];
        y++; if (y >= MAX_HEIGHT) {
          y = 0;
        }
      }


    } // countdn
  }// loop
  for (char x = 0; x < 16; x++) {
    for (char y = 0; y < MAX_HEIGHT; y++) {
      char p = rowStart[x] + y;
      if (p >= MAX_HEIGHT) {
        p -= MAX_HEIGHT;
      }
      for (char t = 0; t < 8; t++) {
        arduboy.drawPixel(t + (x * 8), y, snowRow[p] & (1 << t));
      }
    }
  }
}

